/*
 * 쐬: 2007/01/03
 */
package nat.sw.tray.config;

import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import nat.sw.MessageException;
import nat.sw.Messages;
import nat.sw.WallpaperDirectory;
import nat.sw.WallpaperDirectoryCollection;

/**
 * ǎfBNg̐ݒ(WallpaperDirectoryCollection)ɑΉTableModel.
 */
public class WallpaperDirectoryTableModel extends AbstractTableModel
        implements TableModel {

    public static final int ENABLE_COLUMN = 0;
    public static final int USE_SUB_DIRECTORY_COLUMN = 1;
    public static final int PATH_COLUMN = 2;
    public static final int NUMBER_OF_WALLPAPERS_COLUMN = 3;
    
    /** ǎfBNge[u1s\NX */
    private class Row {
        public static final int WAIT_FOR_COUNTING = -1; //҂\l
        public static final int COUNTING = -2; //\l
        public static final int ERROR = -3; //s\l
        
        public int rowIndex; //̍s̃CfbNX
        public WallpaperDirectory wallpaperDirectory; //ǎfBNg
        public int numberOfWallpaper = WAIT_FOR_COUNTING; //ǎt@C

        /** RXgN^ */
        public Row(int rowIndex, WallpaperDirectory wd) {
            this.rowIndex = rowIndex;
            this.wallpaperDirectory = wd;
        }
    }
    
    /**
     * ǎ𐔂XbhB
     * m_rowsWAIT_FOR_COUNTINGRow΁AnumberOfWallpapervZB
     * ASĂRowɂČvZIAnotify܂waitB
     * notifyꂽAĂWAIT_FOR_COUNTINGRownumberOfWallpapeřvZJԂB
     */
    private class CalcNumberOfWallpaperThread extends Thread {
        public CalcNumberOfWallpaperThread() {
            setPriority(MIN_PRIORITY);
            setDaemon(true);
        }
        public void run() {
            while(true) {
                for(Row row : m_rows) {
                    synchronized(row) {
                        if(row.numberOfWallpaper != Row.WAIT_FOR_COUNTING) {
                            continue;
                        }
                        row.numberOfWallpaper = Row.COUNTING;
                        fireTableCellUpdated(row.rowIndex, NUMBER_OF_WALLPAPERS_COLUMN);
                        
                        int numberOfWallpaper;
                        try {
                            numberOfWallpaper = row.wallpaperDirectory.getNumberOfWallpaper();
                        } catch(MessageException e) {
                            numberOfWallpaper = Row.ERROR;
                        }
                        if(row.numberOfWallpaper != Row.WAIT_FOR_COUNTING) {
                            // IƂɁAݒ_CAOŐݒ肪ύXĂꍇ
                            row.numberOfWallpaper = numberOfWallpaper;
                            fireTableCellUpdated(row.rowIndex, NUMBER_OF_WALLPAPERS_COLUMN);
                        }
                    }
                }
                synchronized(this) {
                    try {
                        wait(3 * 1000); //3b҂
                    } catch(InterruptedException e) {
                        // Ȃ
                    }
                }
            }
        }
    }
    
    private WallpaperDirectoryCollection m_collection;
    private List<Row> m_rows;
    private CalcNumberOfWallpaperThread m_calcThread;
    
    /**
     * RXgN^B
     * @param collection
     */
    public WallpaperDirectoryTableModel(
            WallpaperDirectoryCollection collection) {
        m_collection = collection;
        m_rows = new LinkedList<Row>();
        for(int i = 0; i < collection.getWallpaperDirectory().length; i++) {
            m_rows.add(new Row(i, collection.getWallpaperDirectory()[i]));
        }
        m_calcThread = new CalcNumberOfWallpaperThread();
        m_calcThread.start();
    }
    
    public int getColumnCount() {
        return 4;
    }

    public int getRowCount() {
        return m_rows.size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Row row = m_rows.get(rowIndex);
        switch(columnIndex) {
        case ENABLE_COLUMN: //enable
            return Boolean.valueOf(row.wallpaperDirectory.isEnable());
        case USE_SUB_DIRECTORY_COLUMN: //useSubDirectory
            return Boolean.valueOf(row.wallpaperDirectory.isUseSubDirectory());
        case PATH_COLUMN: //path
            return row.wallpaperDirectory.getPath();
        case NUMBER_OF_WALLPAPERS_COLUMN:
            switch(row.numberOfWallpaper) {
            case Row.WAIT_FOR_COUNTING:
                return Messages.getString("WallpaperDirectoryTableModel.numberOfWallpaper.waitForCounting"); //$NON-NLS-1$
            case Row.COUNTING:
                return Messages.getString("WallpaperDirectoryTableModel.numberOfWallpaper.counting"); //$NON-NLS-1$
            case Row.ERROR:
                return Messages.getString("WallpaperDirectoryTableModel.numberOfWallpaper.error"); //$NON-NLS-1$
            default:
                return row.numberOfWallpaper;
            }
        }
        return null;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName(int column) {
        switch(column) {
        case ENABLE_COLUMN:
            return Messages.getString("WallpaperDirectoryTableModel.header.ebable"); //$NON-NLS-1$
        case USE_SUB_DIRECTORY_COLUMN:
            return Messages.getString("WallpaperDirectoryTableModel.header.useSubDirectory"); //$NON-NLS-1$
        case PATH_COLUMN:
            return Messages.getString("WallpaperDirectoryTableModel.header.path"); //$NON-NLS-1$
        case NUMBER_OF_WALLPAPERS_COLUMN:
            return Messages.getString("WallpaperDirectoryTableModel.header.numberOfWallpaper"); //$NON-NLS-1$
        }
        return ""; //$NON-NLS-1$
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
     */
    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch(columnIndex) {
        case ENABLE_COLUMN:
            return Boolean.class;
        case USE_SUB_DIRECTORY_COLUMN:
            return Boolean.class;
        case PATH_COLUMN:
            return String.class;
        case NUMBER_OF_WALLPAPERS_COLUMN:
            return Number.class;
        }
        return null;
    }
    
    /**
     * indexŎw肵ʒuɕǎtH_ǉB
     * @param index 0n܂Œǉʒuw肷
     * @param wd
     */
    public void add(int index, WallpaperDirectory wd) {
        m_rows.add(index, new Row(index, wd));
        updateRowIndexAndCollection();
        fireTableRowsInserted(index, index);
        notifyCalcThread();
    }

    /**
     * ǎt@CvZXbh֒ʒmB
     */
    private void notifyCalcThread() {
        synchronized(m_calcThread) {
            m_calcThread.notify(); // ǎvZXbhւ̒ʒm
        }
    }

    /**
     * m_rowsɊ܂܂RowrowIndexXVA
     * m_collectionwallpaperDirectoryXVB
     */
    private void updateRowIndexAndCollection() {
        Iterator<Row> itr = m_rows.iterator();
        int index = 0;
        while(itr.hasNext()) {
            itr.next().rowIndex = index++;
        }
        WallpaperDirectory[] wds = new WallpaperDirectory[m_rows.size()];
        itr = m_rows.iterator();
        for(int i = 0; i < wds.length; i++) {
            wds[i] = itr.next().wallpaperDirectory;
        }
        m_collection.setWallpaperDirectory(wds);
    }
    
    /**
     * indexesŎw肵ʒu̕ǎtH_폜B
     * @param indexes 0n܂ō폜ʒuw肷CfbNX̔z
     * @param wd
     */
    public void remove(int[] indexes) {
        if(indexes == null || indexes.length == 0) {
            return; // ̔zȂ牽Ȃ
        }
        
        int maxIndex = Integer.MIN_VALUE;
        int minIndex = Integer.MAX_VALUE;
        for(int index : indexes) {
            // rowIndex == index 藧Row폜
            Iterator<Row> itr = m_rows.iterator();
            while(itr.hasNext()) {
                if(itr.next().rowIndex == index) {
                    itr.remove();
                    break;
                }
            }
            if(maxIndex < index) {
                maxIndex = index;
            }
            if(index < minIndex) {
                minIndex = index;
            }
        }
        updateRowIndexAndCollection();
        fireTableRowsDeleted(minIndex, maxIndex);
    }
    

    /**
     * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
     */
    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        if(columnIndex != 2) {
            return true;
        }
        return false;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object, int, int)
     */
    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
//        System.err.println("\"" + aValue.toString() + "\" @ (" +
//            rowIndex + ", " + columnIndex + ")");
        Row row = m_rows.get(rowIndex);
        switch(columnIndex) {
        case ENABLE_COLUMN:
            row.wallpaperDirectory.setEnable(((Boolean)aValue).booleanValue());
            synchronized(row) {
                row.numberOfWallpaper = Row.WAIT_FOR_COUNTING;
            }
            fireTableCellUpdated(rowIndex, NUMBER_OF_WALLPAPERS_COLUMN);
            notifyCalcThread();
            break;
        case USE_SUB_DIRECTORY_COLUMN:
            row.wallpaperDirectory.setUseSubDirectory(((Boolean)aValue).booleanValue());
            synchronized(row) {
                row.numberOfWallpaper = Row.WAIT_FOR_COUNTING;
            }
            fireTableCellUpdated(rowIndex, NUMBER_OF_WALLPAPERS_COLUMN);
            notifyCalcThread();
            break;
        case PATH_COLUMN:
            row.wallpaperDirectory.setPath(((File)aValue).toString());
            break;
        default:
            return; // 
        }
        fireTableCellUpdated(rowIndex, columnIndex);
        return;
    }

    
}
