/*
 * 쐬: 2007/01/07
 */
package nat.sw.tray.config;

import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.JTableHeader;

import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import nat.sw.Messages;
import nat.sw.WallpaperDirectory;
import nat.sw.WallpaperDirectoryCollection;
import java.awt.GridBagLayout;
import javax.swing.JButton;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.MouseEvent;

import javax.swing.JFileChooser;

public class WallpaperDirectoryTablePanel extends JPanel {

    private static final long serialVersionUID = 1L;
    
    private static final String[] headerTooltips = {
        Messages.getString("WallpaperDirectoryTablePanel.enableHeaderTooltip"), //$NON-NLS-1$
        Messages.getString("WallpaperDirectoryTablePanel.useSubDirectoryHearTooltip"), //$NON-NLS-1$
        Messages.getString("WallpaperDirectoryTablePanel.directoryHeaderTooltip"), //$NON-NLS-1$
        Messages.getString("WallpaperDirectoryTablePanel.numberOfWallpapersHeaderTooltip") //$NON-NLS-1$
    };
    
    private JScrollPane tableScrollPane = null;
    private JTable table = null;
    private WallpaperDirectoryCollection m_wallpaperDirectoryCollection;
    private WallpaperDirectoryTableModel m_tableModel;
    private JPanel controlPanel = null;
    private JButton removeButton = null;
    private JButton addButton = null;
    private JFileChooser fileChooser = null;
    
    /**
     * This is the default constructor
     */
    public WallpaperDirectoryTablePanel() {
        this(new WallpaperDirectoryCollection());
    }
    
    public WallpaperDirectoryTablePanel(
            WallpaperDirectoryCollection wallpaperDirectoryCollection) {
        super();
        setWallpaperDirectoryCollection(wallpaperDirectoryCollection);
        initialize();
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        this.setLayout(new BorderLayout());
        this.setSize(new Dimension(500, 400));
        this.add(getTableScrollPane(), BorderLayout.CENTER);
        this.add(getControlPanel(), BorderLayout.SOUTH);
    }

    /**
     * This method initializes tableScrollPane	
     * 	
     * @return javax.swing.JScrollPane	
     */
    private JScrollPane getTableScrollPane() {
        if (tableScrollPane == null) {
            tableScrollPane = new JScrollPane();
            tableScrollPane.setBorder(null);
            tableScrollPane.setPreferredSize(new Dimension(520, 150));
            tableScrollPane.setViewportView(getTable());
        }
        return tableScrollPane;
    }

    /**
     * This method initializes table	
     * 	
     * @return javax.swing.JTable	
     */
    private JTable getTable() {
        if (table == null) {
            table = new JTable() {
                /**
                 * wb_̃c[`bvݒpɏ㏑
                 * @see javax.swing.JTable#createDefaultTableHeader()
                 */
                @Override
                protected JTableHeader createDefaultTableHeader() {
                    return new JTableHeader(columnModel) {
                        public String getToolTipText(MouseEvent e) {
                            java.awt.Point p = e.getPoint();
                            int index = columnModel.getColumnIndexAtX(p.x);
                            int realIndex = 
                                    columnModel.getColumn(index).getModelIndex();
                            return headerTooltips[realIndex];
                        }
                    };
                }
            };
            table.setAutoCreateColumnsFromModel(false);
            
            table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    // sIԂɉč폜{^̏ԂύX
                    if(getTable().getSelectedRowCount() == 0) {
                        getRemoveButton().setEnabled(false);
                    } else {
                        getRemoveButton().setEnabled(true);
                    }
                }
                
            });
        }
        return table;
    }

    /**
     * wallpaperDirectoryCollection ԂB
     * @return wallpaperDirectoryCollection
     */
    public WallpaperDirectoryCollection getWallpaperDirectoryCollection() {
        return m_wallpaperDirectoryCollection;
    }

    /**
     * wallpaperDirectoryCollection ݒ肷B
     * @param wallpaperDirectoryCollection wallpaperDirectoryCollection
     */
    public void setWallpaperDirectoryCollection(
            WallpaperDirectoryCollection wallpaperDirectoryCollection) {
        m_wallpaperDirectoryCollection = wallpaperDirectoryCollection;
        m_tableModel = new WallpaperDirectoryTableModel(
                m_wallpaperDirectoryCollection);
        getTable().setModel(m_tableModel);
        getTable().createDefaultColumnsFromModel();
        getTable().getColumnModel().getColumn(2).
            setCellEditor(new DirectoryCellEditor());
        final int width[] = {50, 80, 300, 90};
        for(int i = 0; i < width.length; i++) {
            table.getColumnModel().getColumn(i).
                setPreferredWidth(width[i]);
        }
    }

    /**
     * This method initializes controlPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getControlPanel() {
        if (controlPanel == null) {
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridx = 1;
            gridBagConstraints1.insets = new Insets(0, 20, 0, 0);
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.insets = new Insets(0, 0, 0, 20);
            gridBagConstraints.gridy = 0;
            controlPanel = new JPanel();
            controlPanel.setLayout(new GridBagLayout());
            controlPanel.add(getRemoveButton(), gridBagConstraints1);
            controlPanel.add(getAddButton(), gridBagConstraints);
        }
        return controlPanel;
    }

    /**
     * This method initializes removeButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getRemoveButton() {
        if (removeButton == null) {
            removeButton = new JButton();
            removeButton.setText(Messages.getString("WallpaperDirectoryTablePanel.deleteButton")); //$NON-NLS-1$
            removeButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    removeSelectedWallpaperDirectory();
                }
            });
        }
        return removeButton;
    }

    /**
     * This method initializes addButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getAddButton() {
        if (addButton == null) {
            addButton = new JButton();
            addButton.setText(Messages.getString("WallpaperDirectoryTablePanel.addButton")); //$NON-NLS-1$
            addButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    addNewWallpaperDirectory();
                }
            });
        }
        return addButton;
    }

    /**
     * VKɕǎfBNgǉ
     */
    public void addNewWallpaperDirectory() {
        int option = getFileChooser().showDialog(
                this,
                Messages.getString("ConfigDialog.fileField.select")); //$NON-NLS-1$
        if(option == JFileChooser.APPROVE_OPTION) {
            WallpaperDirectory wd = new WallpaperDirectory();
            wd.setEnable(true);
            wd.setUseSubDirectory(false);
            wd.setPath(getFileChooser().getSelectedFile().toString());
            
            int index = table.getSelectedRow();
            if(index < 0) {
                index = 0;
            }
            m_tableModel.add(index, wd);
        }
    }

    /**
     * IꂽǎfBNg폜
     */
    public void removeSelectedWallpaperDirectory() {
        m_tableModel.remove(table.getSelectedRows());
    }

    /**
     * This method initializes fileChooser	
     * 	
     * @return javax.swing.JFileChooser	
     */
    private JFileChooser getFileChooser() {
        if (fileChooser == null) {
            fileChooser = new JFileChooser();
            fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        }
        return fileChooser;
    }
}
